SystemOrganization addCategory: #'EjerciciosPoo-TusLibros'!

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 19:58'!
clientInterfaceForTesting

	^ClientInterface new initializeWithPriceList: Dictionary new andCashier: (Cashier new initializeWith: self dummyMerchantProcessorInterface)! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 17:30'!
dummyCashier

	^Cashier new initializeWith: self dummyMerchantProcessorInterface! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:30'!
dummyMerchantProcessorInterface

	^MerchantProcessorInterfaceSimulator new initializeWith: [ ]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/4/2013 19:32'!
expiredCreditCard

	^CreditCard new initializeWith: 123 expirationDate: (Date today subtractDays: 120) andOwner: 'Juan Gomez'! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/4/2013 19:43'!
nonExpiredCreditCard

	^CreditCard new initializeWith: 123 expirationDate: (Date today addDays: 120) andOwner: 'Juan Gomez'! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:30'!
stolenCardMerchantProcessorInterface

	^MerchantProcessorInterfaceSimulator new initializeWith: [ Error signal: MerchantProcessorInterfaceSimulator stolenCreditCardErrorMessage ]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:09'!
test01CartShouldBeEmptyAfterCreation

	| cart priceList |
	
	priceList := Dictionary new.
	cart := Cart new initializeWith: priceList.
	self assert: cart isEmpty! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 17:28'!
test02CartShouldNotBeEmptyAfterAddingABook

	| cart priceList isbn |
	
	isbn := 123.
	priceList := Dictionary new.
	priceList at: isbn put: 5.
	cart := Cart new initializeWith: priceList.
	cart add: isbn.
	
	self deny: cart isEmpty! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 17:28'!
test03CartShouldNotAllowToAddABookWhichIsNotInTheCatalog

	| cart isbn priceList |

	isbn :=  123.
	priceList := Dictionary new.	
	cart := Cart new initializeWith: priceList.
	
	self should: [ cart add: isbn ] raise: Error withExceptionDo: [ :anError | 
		self assert: anError messageText = Cart itemNotInCatalogErrorMessage.
		self assert: cart isEmpty
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 17:29'!
test04AbookThatIsAddedToTheCartIsInTheCart

	| cart priceList isbn |

	isbn := 123.
	priceList := Dictionary new.
	priceList at: isbn put: 5.
	cart := Cart new initializeWith: priceList.
	cart add: isbn.
	
	self assert: (cart includes: isbn)! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 20:59'!
test05CannotCheckoutAnEmptyCart

	| cart priceList cashier creditCard salesBookSize |

	priceList := Dictionary new.
	cart := Cart new initializeWith: priceList.
	creditCard := CreditCard new.
	cashier := Cashier new initializeWith: self dummyMerchantProcessorInterface.
	salesBookSize := cashier salesBook size.
	
	self should: [ cashier checkout: cart with: creditCard ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = Cashier emptyCartErrorMessage.
		self assert: salesBookSize = cashier salesBook size
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 17:30'!
test06CheckoutWithOneItemGeneratesASaleWithTheSamePriceAsTheItem

	| cart priceList cashier isbn creditCard |

	isbn := 123.
	priceList := Dictionary new.
	priceList at: isbn put: 5.
	cart := Cart new initializeWith: priceList.
	cart add: isbn.
	cashier := Cashier new initializeWith: self dummyMerchantProcessorInterface.
	creditCard := self nonExpiredCreditCard.
	
	self assert: ((cashier checkout: cart with: creditCard) total = (priceList at: isbn))! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 17:35'!
test07CheckoutWithManyItemsGeneratesASaleWithTheSumOfTheItemsPricesAsTotal

	| cart priceList cashier anIsbn anotherIsbn creditCard |

	anIsbn := 123.
	anotherIsbn := 321.
	priceList := Dictionary newFromPairs: (Array with: anIsbn with: 5 with: anotherIsbn with: 8).
	cart := Cart new initializeWith: priceList.
	cart add: anIsbn.
	cart add: anotherIsbn.
	cashier := Cashier new initializeWith: self dummyMerchantProcessorInterface.
	creditCard := self nonExpiredCreditCard.
	
	self assert: ((cashier checkout: cart with: creditCard) total = ((priceList at: anIsbn) + (priceList at: anotherIsbn)) )! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 17:36'!
test08CheckoutOfAnItemGeneratesASaleThatIncludesTheItem

	| cart priceList cashier isbn creditCard |

	isbn := 123.
	priceList := Dictionary newFromPairs: (Array with: isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: isbn.
	cashier := Cashier new initializeWith: self dummyMerchantProcessorInterface.
	creditCard := self nonExpiredCreditCard.
	
	self assert: ((cashier checkout: cart with: creditCard) includesBook: isbn)! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 17:36'!
test09ChangesInPriceListDontAffectOlderSalesPrices

	| cart priceList cashier isbn saleTotal creditCard |

	isbn := 123.
	priceList := Dictionary newFromPairs: (Array with: isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: isbn.
	cashier := Cashier new initializeWith: self dummyMerchantProcessorInterface.
	creditCard := self nonExpiredCreditCard.
	saleTotal := (cashier checkout: cart with: creditCard) total.
	priceList at: isbn put: 8.
	
	self assert: (saleTotal = cashier salesBook last total )! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 17:36'!
test10CannotCheckoutWithExpiredCreditCard

	| cart priceList cashier isbn creditCard salesBookSize |

	isbn := 123.
	priceList := Dictionary newFromPairs: (Array with: isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: isbn.
	cashier := Cashier new initializeWith: MerchantProcessorInterfaceSimulator new.
	salesBookSize := cashier salesBook size.
	creditCard := self expiredCreditCard.
	
	self should: [ cashier checkout: cart with: creditCard ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = CreditCard expiredCardErrorMessage.
		self assert: salesBookSize = cashier salesBook size
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 17:37'!
test11CannotCheckoutWithStolenCreditCard

	| cart priceList cashier isbn creditCard salesBookSize |

	isbn := 123.
	priceList := Dictionary newFromPairs: (Array with: isbn with: 5).
	cart := Cart new initializeWith: priceList.
	cart add: isbn.
	cashier := Cashier new initializeWith: self stolenCardMerchantProcessorInterface.
	salesBookSize := cashier salesBook size.
	creditCard := self nonExpiredCreditCard.
	
	self should: [ cashier checkout: cart with: creditCard ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = MerchantProcessorInterface stolenCreditCardErrorMessage.
		self assert: salesBookSize = cashier salesBook size
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 19:57'!
test12InvalidClientIdCannotCreateCart

	| clientInterface |
	
	clientInterface := self clientInterfaceForTesting.
	self should: [ clientInterface createCartForClientId: 1 withPassword: 'abc' ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = ClientInterface invalidClientIdErrorMessage.
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 19:57'!
test13ValidClientIdWithInvalidPasswordCannotCreateCart

	| clientInterface |
	
	clientInterface := self clientInterfaceForTesting.
	clientInterface addClientId: 1 withPassword: 'abc'.
	self should: [ clientInterface createCartForClientId: 1 withPassword: 'abz' ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = ClientInterface invalidPasswordErrorMessage.
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 19:57'!
test14ValidClientGetsCartIdAfterCreateCart

	| clientInterface |
	
	clientInterface := self clientInterfaceForTesting.
	clientInterface addClientId: 1 withPassword: 'abc'.
	self assert: (clientInterface isInCurrentCarts: (clientInterface createCartForClientId: 1 withPassword: 'abc'))! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 19:57'!
test15CartCreatedByValidClientIsEmptyAfterCreation

	| clientInterface cartId |
	
	clientInterface := self clientInterfaceForTesting.
	clientInterface addClientId: 1 withPassword: 'abc'.
	cartId :=  clientInterface createCartForClientId: 1 withPassword: 'abc'.
	self assert: (clientInterface listCart: cartId) isEmpty! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 19:57'!
test16ValidClientCannotAddBookThatIsNotInThePriceList

	| clientInterface cartId isbn |
	
	isbn := 123.
	clientInterface := self clientInterfaceForTesting.
	clientInterface addClientId: 1 withPassword: 'abc'.
	cartId :=  clientInterface createCartForClientId: 1 withPassword: 'abc'.
	self should: [ clientInterface addBook: isbn quantity: 1 toCart: cartId ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = Cart itemNotInCatalogErrorMessage.
		self assert: (clientInterface listCart: cartId) isEmpty
	]

! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 17:31'!
test17BookAddedByValidClientIsIncludedInHisCart

	| priceList isbn clientInterface cartId |

	isbn := 123.
	priceList := Dictionary new.
	priceList at: isbn put: 5.
	clientInterface := ClientInterface new initializeWithPriceList: priceList andCashier: self dummyCashier.
	clientInterface addClientId: 1 withPassword: 'abc'.
	cartId :=  clientInterface createCartForClientId: 1 withPassword: 'abc'.
	clientInterface addBook: isbn quantity: 1 toCart: cartId.
	
	self assert: (((clientInterface listCart: cartId) at: isbn) = 1)

! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 17:31'!
test18BookAddedQuantityGreaterThanOneIsIncludedInCartTheGivenQuantity

	| priceList isbn clientInterface cartId |

	isbn := 123.
	priceList := Dictionary new.
	priceList at: isbn put: 5.
	clientInterface := ClientInterface new initializeWithPriceList: priceList andCashier: self dummyCashier.
	clientInterface addClientId: 1 withPassword: 'abc'.
	cartId :=  clientInterface createCartForClientId: 1 withPassword: 'abc'.
	clientInterface addBook: isbn quantity: 3 toCart: cartId.
	
	self assert: (((clientInterface listCart: cartId) at: isbn) = 3)

! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 17:30'!
test19SameBookAddedManyTimesIsIncludedInCartWithTheCorrectQuantity

	| priceList isbn clientInterface cartId |

	isbn := 123.
	priceList := Dictionary new.
	priceList at: isbn put: 5.
	clientInterface := ClientInterface new initializeWithPriceList: priceList andCashier: self dummyCashier.
	clientInterface addClientId: 1 withPassword: 'abc'.
	cartId :=  clientInterface createCartForClientId: 1 withPassword: 'abc'.
	clientInterface addBook: isbn quantity: 1 toCart: cartId.
	clientInterface addBook: isbn quantity: 1 toCart: cartId.
	clientInterface addBook: isbn quantity: 1 toCart: cartId.
	
	self assert: (((clientInterface listCart: cartId) at: isbn) = 3)

! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 17:30'!
test20TwoDifferentBooksAddedAreIncludedInCartCorrectly

	| priceList anIsbn anotherIsbn clientInterface cartId |

	anIsbn := 123.
	anotherIsbn := 345.
	priceList := Dictionary new.
	priceList at: anIsbn put: 5. priceList at: anotherIsbn put: 7.
	clientInterface := ClientInterface new initializeWithPriceList: priceList andCashier: self dummyCashier.
	clientInterface addClientId: 1 withPassword: 'abc'.
	cartId :=  clientInterface createCartForClientId: 1 withPassword: 'abc'.
	clientInterface addBook: anIsbn quantity: 1 toCart: cartId.
	clientInterface addBook: anotherIsbn quantity: 1 toCart: cartId.
	
	self assert: (((clientInterface listCart: cartId) at: anIsbn) = 1).
	self assert: (((clientInterface listCart: cartId) at: anotherIsbn) = 1)

! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 18:34'!
test21ClientCannotCheckoutAnEmptyCart

	| clientInterface cartId |

	clientInterface := self clientInterfaceForTesting.
	clientInterface addClientId: 1 withPassword: 'abc'.
	cartId :=  clientInterface createCartForClientId: 1 withPassword: 'abc'.
	
	self should: [ clientInterface checkout: cartId withCreditCardNumber: 546 andExpirationDate: Date today andOwner: 'Pepe' ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = Cashier emptyCartErrorMessage.
		self assert: (clientInterface listPurchasesForClient: 1 withPassword: 'abc') = 0
	]! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 19:26'!
test22ClientCannotAddBookToCartAfter30MinutesIdle

	| priceList isbn clientInterface cartId aClock |

	isbn := 123.
	priceList := Dictionary new.
	priceList at: isbn put: 5.
	aClock := DateAndTime now.
	clientInterface := ClientInterface new initializeWithPriceList: priceList andCashier: self dummyCashier andClock: aClock.
	clientInterface addClientId: 1 withPassword: 'abc'.
	cartId :=  clientInterface createCartForClientId: 1 withPassword: 'abc'.
	aClock := aClock + (Duration minutes: 30).
	
	self should: [ clientInterface addBook: isbn quantity: 1 toCart: cartId. ] raise: Error withExceptionDo: [ :anError |
		self assert: anError messageText = ClientInterface sessionExpiredErrorMessage.
		self assert: (clientInterface listCart: cartId) isEmpty
	]! !

Object subclass: #Cart
	instanceVariableNames: 'booksInCart priceList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 20:38'!
itemNotInCatalogErrorMessage

	^'El item no se encuentra en el catlogo'! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 17:13'!
add: aBookIsbn

	^(priceList includesKey: aBookIsbn) ifTrue: [ booksInCart add: aBookIsbn ] ifFalse: [ Error signal: self class itemNotInCatalogErrorMessage ]! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:20'!
booksInCart

	^booksInCart! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 20:51'!
includes: aBook

	^booksInCart includes: aBook! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:11'!
initializeWith: aPriceList

	booksInCart := Bag new.
	priceList := aPriceList! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 17:39'!
isEmpty

	^booksInCart isEmpty! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:11'!
priceList

	^priceList! !

!Cart methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 17:34'!
totalAmountToCheckout

	| total |
	
	total := 0.
	booksInCart do: [ :bookIsbn | total := total + (priceList at: bookIsbn) ].
	^total

! !

Object subclass: #Cashier
	instanceVariableNames: 'salesBook mpi'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 5/29/2013 21:06'!
emptyCartErrorMessage

	^'El carrito est vacio'! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 17:58'!
checkout: aCart with: aCreditCard

	| newSale bookWithPrice |

	aCart isEmpty ifTrue: [ Error signal: self class emptyCartErrorMessage ] ifFalse: [
		(aCreditCard expiredOn: (Date today)) ifTrue: [ Error signal: aCreditCard class expiredCardErrorMessage ].
		mpi debit: aCart totalAmountToCheckout from: aCreditCard.
		newSale := Sale new.
		aCart booksInCart do: [ :bookIsbn | bookWithPrice := Association key: bookIsbn value: (aCart priceList at: bookIsbn). newSale add: bookWithPrice ].
		salesBook add: newSale.
		^newSale
	]! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 20:49'!
initializeWith: aMerchantProcessorInterface

	salesBook := OrderedCollection new.
	mpi := aMerchantProcessorInterface! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 20:24'!
salesBook

	^salesBook! !

Object subclass: #ClientInterface
	instanceVariableNames: 'validClients currentCarts cartsClients clientsSales priceList cashier clock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!ClientInterface class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:36'!
invalidClientIdErrorMessage

	^'El id de cliente es invlido'! !

!ClientInterface class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/9/2013 20:08'!
invalidPasswordErrorMessage

	^'El password es invlido'! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 18:29'!
addBook: aBookIsbn quantity: aQuantity toCart: aCartId

	| cart c |
	
	cart := currentCarts at: aCartId.
	c := 0.
	^[ c < aQuantity ] whileTrue: [ cart add: aBookIsbn. c := c+1 ]! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 18:01'!
addClientId: aClientId withPassword: aPassword

	^validClients at: aClientId put: aPassword! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 18:25'!
checkout: aCartId withCreditCardNumber: aCreditCardNumber andExpirationDate: aDate andOwner: aName

	| cart creditCard sale |
	
	cart := currentCarts at: aCartId.
	creditCard := CreditCard new initializeWith: aCreditCardNumber expirationDate: aDate andOwner: aName.
	sale := cashier checkout: cart with: creditCard.
	(clientsSales at: (self clientOfCart: aCartId) ifAbsentPut: [ OrderedCollection with: sale ]) add: sale.
	^sale! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 18:25'!
clientOfCart: aCartId

	^cartsClients at: aCartId! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 18:35'!
createCartForClientId: aClientId withPassword: aPassword

	| cartId |
	
	(self validateClientId: aClientId withPassword: aPassword) ifTrue: [ 
		cartId := self newCartId.
		currentCarts at: cartId put: (Cart new initializeWith: priceList).
		cartsClients at: cartId put: aClientId.
		^cartId
	]ifFalse: [ 
		Error signal: self class invalidPasswordErrorMessage
	]! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 19:25'!
initializeWithPriceList: aPriceList andCashier: aCashier

	validClients := Dictionary new.
	currentCarts := Dictionary new.
	cartsClients  := Dictionary new.
	clientsSales := Dictionary new.
	priceList := aPriceList.
	cashier := aCashier.
	clock := DateAndTime now! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 19:25'!
initializeWithPriceList: aPriceList andCashier: aCashier andClock: aClock

	validClients := Dictionary new.
	currentCarts := Dictionary new.
	cartsClients  := Dictionary new.
	clientsSales := Dictionary new.
	priceList := aPriceList.
	cashier := aCashier.
	clock := aClock! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:28'!
isInCurrentCarts: aCartId

	^currentCarts includesKey: aCartId! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 18:23'!
listCart: aCartId

	| cart |
	
	cart := currentCarts at: aCartId.
	^cart booksInCart valuesAndCounts! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 18:41'!
listPurchasesForClient: aClientId withPassword: aPassword

	| totalAmount |
	
	totalAmount := 0.
	"Pendiente que devuelva el listado de libros con cantidad"
	clientsSales at: aClientId ifPresent: [ :seq | seq do: [ :sale | sale items. totalAmount := totalAmount + sale total. ] ].
	^totalAmount! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/10/2013 00:29'!
newCartId

	^Random new nextInt: 999999! !

!ClientInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 18:01'!
validateClientId: aClientId withPassword: aPassword

	^(validClients at: aClientId ifAbsent: [ Error signal: self class invalidClientIdErrorMessage ] ) = aPassword! !

Object subclass: #CreditCard
	instanceVariableNames: 'number expirationDate owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!CreditCard class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 20:42'!
expiredCardErrorMessage

	^'La tarjeta se encuentra vencida'! !

!CreditCard methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:04'!
expiredOn: aDate

	^expirationDate < aDate! !

!CreditCard methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:21'!
initializeWith: aNumber expirationDate: anExpirationDate andOwner: anOwner

	number := aNumber.
	expirationDate := anExpirationDate.
	owner := anOwner! !

Object subclass: #MerchantProcessorInterface
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!MerchantProcessorInterface class methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 21:28'!
stolenCreditCardErrorMessage

	^'La tarjeta es robada'! !

!MerchantProcessorInterface methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/4/2013 19:15'!
debit: anAmount from: aCreditCard

	^self subclassResponsibility! !

MerchantProcessorInterface subclass: #MerchantProcessorInterfaceSimulator
	instanceVariableNames: 'debitFromAction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!MerchantProcessorInterfaceSimulator methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 20:16'!
debit: anAmount from: aCreditCard

	^debitFromAction value! !

!MerchantProcessorInterfaceSimulator methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/5/2013 20:51'!
initializeWith: aDebitFromAction

	debitFromAction := aDebitFromAction! !

Object subclass: #Sale
	instanceVariableNames: 'items'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:40'!
add: anItem

	items add: anItem.! !

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/3/2013 18:26'!
includesBook: anIsbn

	^items includesKey: anIsbn! !

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:38'!
initialize

	items := Dictionary new.! !

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/12/2013 18:28'!
items

	^items keys! !

!Sale methodsFor: 'as yet unclassified' stamp: 'AgustinMosteiro 6/2/2013 19:44'!
total

	| total |
	
	total := 0.
	items do: [ :item | total := total + item value ].
	^total! !
